
/*************************/
/*** various utilities ***/
/*************************/

//char const ndxtbl[10] =
//{ 8, 9, 0, 1, 2, 3, 4, 5, 6, 7 };

/*********************** prototypes ********************************/

void watchdog(void)
{
    /* reset PIC watchdog timer */
    if (b_wdt)
    { restart_wdt();
      b_wdt = 0;
    }
}
/*******************************************************************/

void bin2bcd(char astr[], char num)
{ // convert a binary no. to 2 ASCII digits representing HEX in <astr>
  // e.g. 0x4f -> ['4','F']

  char c1;

    c1 = num/16 + '0';
    if (c1 > '9')
      c1 += 7;
    astr[0] = c1;
    c1 = (num & 0x0f) + '0';
    if (c1 > '9')
      c1 += 7;
    astr[1] = c1;
}
/*********************************************************************/

char int2asc(int16 inum, char astr[], char ndig)
{ /* convert the int16 number to (1 -> 5) ASCII's */
  /* eg: 1025 -> str[pstr..] = " 1025" */
  // returns no.of ASCII digits
  // <ndig & 0x07> = returns this max.no.of digits, filling leader with '0' for <astr[0]>
  // bit <ndig,7> means leader = 'astr[0]' instead of '0' or nothing

  char c1, n0, n1, n2, n3, cleadr;
  int16 i16;

    n0 = 0;
    if (bit_test(ndig,7))
    { cleadr = astr[0];
      n0++;
    }
    else
      cleadr = '0';
    ndig &= 0x07;

    n2 = 0;
    if (inum & 0x8000)
    { astr[0] = '-';
      n2++;
      inum ^= 0xffff;
      inum++;
    }

    if (n0)
    { if (ndig == 4)
      { if (inum > 9999)
          inum = 9999;
      }
      else if (ndig == 3)
      { if (inum > 999)
          inum = 999;
      }
      else if (ndig == 2)
      { if (inum > 99)
          inum = 99;
      }
      else if (ndig == 1)
      { if (inum > 9)
          inum = 9;
      }
    }

    n0 = 0;
    n3 = 5;
    c1 = 0;
    while (n0 < 4)
    { n1 = 0;
      if (n0 == 0)
        i16 = 10000;
      else if (n0 == 1)
        i16 = 1000;
      else if (n0 == 2)
        i16 = 100;
      else if (n0 == 3)
        i16 = 10;

      while (inum >= i16)
      { n1++;
        inum -= i16;
      }
      if (n1)
      { astr[n2] = n1 + '0';
        n2++;
        c1 = '0';
      }
      else if (c1)
      { astr[n2] = '0';
        n2++;
      }
      else if (n3 <= ndig)
      { astr[n2] = cleadr;
        n2++;
      }
      n0++;
      n3--;
    }

    astr[n2] = (char)inum + '0';
    n2++;

    return(n2);
}
/**********************************************************************/
/**********************************************************************/

#if code_d2 == 1

void tx_1(void)
{ // check tx buffer and tx if UART is ready

  #byte txreg1  = TXREG1
  #byte txsta1  = TXSTA1

    if (tx_in != tx_out)                /* check for bytes to tx */
    { if (bit_test(txsta1,TRMT))         /* check if UART ready */
      { txreg1 = tx[tx_out];            // uses ind.reg.<INDF0>
        tx_out++;
        if (tx_out >= n_tx)
          tx_out = 0;
      }
    }
}
/*******************************************************************/

void tx_byte(char tbyte)
{
  char txin;

  #byte txreg1  = TXREG1
  #byte txsta1  = TXSTA1

    /* put a byte into the Tx buffer for RS232 transmission */
    /* if queue is full it will wait until UART has room to tx */

    /* advance the buffer pointer and check it */
    txin = tx_in;
    txin++;
    if (txin >= n_tx)
      txin = 0;

    if (txin == tx_out)
    { // queue is full - send one away
      while (!bit_test(txsta1,TRMT));   // stay here until there's room in the UART for tx'ing
      txreg1 = tx[tx_out];              // uses ind.reg.<INDF0>
      tx_out++;
      if (tx_out >= n_tx)
        tx_out = 0;
      n_rxd = 0;
    }

    if (txin != tx_out)
    { tx[tx_in] = tbyte;                // add to the queue
      tx_in = txin;
    }

    tx_1();                             // send it

    watchdog();
}
/**********************************************************************/

void tx_str(char astr[], char len)
{
  char n1;

    n1 = 0;
    while (len)
    { tx_byte(astr[n1]);
      n1++;
      len--;
    }
}
#endif
/*******************************************************************/

void dispint(int16 i1, char ndisp)
{ // accept an int16 and display the digits

  char min;
  char lstr[5];

    min = 0;
    if (i1 & 0x8000)
    { bit_set(min,0);
      i1 = 0 - i1;
    }
    if (i1 >= 1999)
      i1 = 1999;
    if (i1 > 999)
    { i1 -= 1000;
      bit_set(min,1);
    }
    lstr[0] = 0;
//    if (!bit_test(min,1))
//      lstr[1] = ' ';
//    else
    int2asc(i1, lstr+1, 0x03);
    if (bit_test(min,0))
      bit_set(lstr[0],5);
    if (bit_test(min,1))
      bit_set(lstr[0],4);
    bit_set(lstr[0],0);     // decimal point to indicate mode id follows

    switch (ndisp)
    { case 0:
        min = dstr[15] & 0xc0;
        lstr[0] |= min;
        memcpy(dstr+15, lstr, 4);
        dstr[15] &= 0xce;
        dstr[15] |= lstr[0];
        break;

      case 1:
        min = dstr[10] & 0xc0;
        lstr[0] |= min;
        memcpy(dstr+10, lstr, 4);
        dstr[10] &= 0xce;
        dstr[10] |= lstr[0];
        break;

      case 2:
        min = dstr[5] & 0xc0;
        lstr[0] |= min;
        memcpy(dstr+5, lstr, 4);
        dstr[5] &= 0xce;
        dstr[5] |= lstr[0];
        break;

      case 3:
        min = dstr[0] & 0xc0;
        lstr[0] |= min;
        memcpy(dstr+0, lstr, 4);
        dstr[0] &= 0xce;
        dstr[0] |= lstr[0];
        break;
    }
}
/*******************************************************************/
/********************* service port functions **********************/
/*******************************************************************/

#if code_d2 == 1

void rx_msg(char cmdstr[], char nrx)
{ // parse the data coming from RS232

  char n0, n2, n3;
  char ltx[n_tx];
  union Dat2 liu;

    n0 = 0;
    ltx[0] = '#';
    /*** parse the data in <cmdstr> ***/
    cmdstr[nrx] = 0;

    if ((nrx >= 2) && (cmdstr[0] == '#'))
    {   watchdog();

        switch(cmdstr[1])
        {
          case 'V':                     // #V -> S/W version
            ltx[1] = c_version;
            ltx[2] = c_date1;
            ltx[3] = c_date2;
            ltx[4] = c_date3;
            n0 = 5;
            break;

          case 's':                     // #s -> read <dstr>
            n0 = 20;
            memcpy(ltx, dstr, n0);
            break;

          case 'D':                     // #Dctttts -> displ.str.<tttt> and symbols<s> on LCD no.<c>
            if (nrx >= 7)
            { if (cmdstr[2] == '1')         // = <c>
              { memcpy(dstr+16, cmdstr+3, 4);
                if (nrx >= 8)
                  dstr[15] = cmdstr[7];     // = <s>
                else
                  dstr[15] = 0;
                dispstr(dstr+15, 0);
              }
              else if (cmdstr[2] == '2')    // = <c>
              { memcpy(dstr+11, cmdstr+3, 4);
                if (nrx >= 8)
                  dstr[10] = cmdstr[7];
                else
                  dstr[10] = 0;
                dispstr(dstr+10, 1);
              }
              else if (cmdstr[2] == '3')    // = <t...>
              { memcpy(dstr+6, cmdstr+3, 4);
                if (nrx >= 8)
                  dstr[5] = cmdstr[7];
                else
                  dstr[5] = 0;
                dispstr(dstr+5, 2);
              }
              else if (cmdstr[2] == '4')    // =<.t..>
              { memcpy(dstr+1, cmdstr+3, 4);
                if (nrx >= 8)
                  dstr[0] = cmdstr[7];
                else
                  dstr[0] = 0;
                dispstr(dstr, 3);
              }
              tdisp = 255;
              ltx[1] = cmdstr[1];
              ltx[2] = cmdstr[2];
              n0 = 3;
            }
            break;

          case 'd':                     // #dbbc -> display <bb> as a number on LCD no.<c>
            if ((nrx >= 5) && (cmdstr[4] >= '0') && (cmdstr[4] <= '9'))
            { liu.c[1] = cmdstr[2];
              liu.c[0] = cmdstr[3];
              n2 = cmdstr[4] - '1';
              dispint(liu.i, n2);

              n3 = (3 - n2) * 5;
              dispstr(dstr+n3, n2);

              tdisp = 255;
              ltx[1] = cmdstr[1];
              ltx[2] = cmdstr[2];
              n0 = 3;
            }
            break;

          case 'x':                     // #x -> processor restart via WDT
            b_gieh = 0;
            b_giel = 0;
            for(;;);
            break;
        }
        if (n0)
        { if (n0 >= n_tx)
            n0 = n_tx - 1;
          tx_str(ltx, n0);
        }
        else
        { ltx[0] = '#';
          ltx[1] = '0'+nrx;
          tx_str(ltx, 2);
        }
        watchdog();
    }
}
#endif
/*******************************************************************/

void updateDisplay(void)
{
  char ndx, n1;

    n1 = 0;
    if (bit_test(updlcd,0))
    { ndx = 3;
      n1 = 0;
      bit_clear(updlcd,0);
    }
    else if (bit_test(updlcd,1))
    { ndx = 2;
      n1 = 5;
      bit_clear(updlcd,1);
    }
    else if (bit_test(updlcd,2))
    { ndx = 1;
      n1 = 10;
      bit_clear(updlcd,2);
    }
    else if (bit_test(updlcd,3))
    { ndx = 0;
      n1 = 15;
      bit_clear(updlcd,3);
    }

    dispstr(dstr+n1,ndx);       // display <dstr>
}
/*******************************************************************/

